CRUST_ROCKY_TESTNET_ENDPOINT = "wss://rpc-rocky.crust.network"
CRUST_MAINNET_ENDPOINT = "wss://rpc.crust.network"
CRUST_SHADOW_ENDPOINT = "wss://rpc-shadow.crust.network/"
CRUST_TYPE_REGISTRY_PRESET = "crust"
CRUST_SS_58_FORMAT = 66
